@@ -46,7 +46,9 @@ general = {
     "rInverse()": ["matrix M", "Computes the inverse of a matrix by putting an augmented matrix into RREF using Gauss-Jordan elimination, and returns this inverse."],
     "elimSystem()": ["matrix A, vector x, vector b, num r=1000", "Computes the solutions to the system Ax=b, using Gauss-Jordan elimination. A must be a square matrix of coefficients, x and b should be column vectors in string and numerical formats respectively. r is the rational-denominator limit. The output will be a list, including both the solution matrix and a series of solution strings.", "elimSystem"],
     "isMatrix()": ["obj A", "Returns True if the passed object is a matrix, otherwise returns False."],
-    "nthRegression()": ["num r=0", "Computes an nth-degree regression by the least squares method and returns the best fit polynomial. Printing output restricted if r==1."]
+    "nthRegression()": ["num r=0, matrix A=['empty'], num n=1, num rcomp=1", "Computes an nth-degree regression by the least squares method and returns the best fit polynomial. Printing output restricted if r==1. Matrix input and use of n (degree preset) only intended for use by other dependent regression functions. R^2 coefficient of determination will be computed if rcomp==1."],
+    "pwrRegression()": ["num r=0, num rcomp=1", "Computes a power regression by a logarithmic-linear least squares method and returns the best fitting function y=ax^b. Printing output restricted if r==1. R^2 coefficient of determination will be computed if rcomp==1."],
+    "expRegression()": ["num r=0, num rcomp=1", "Computes an exponential regression by a logarithmic-linear least squares method and returns the best fitting function y=ab^x. Printing output restricted if r==1. R^2 coefficient of determination will be computed if rcomp==1."]
 }
 
 
@@ -714,9 +716,9 @@ def isMatrix(obj):
     else:
         return False
 
-def nthRegression(restrict=0,inMatrix=["empty"],ndeg=0):
+def nthRegression(restrict=0,inMatrix=["empty"],ndeg=1,rcomp=1):
     # Performs least-squares regression in order to find the best fitting nth-degree polynomial. For n > 6 the computation may take a long time / not work, as computational expense goes up exponentially with increasing n (inverse of (n+1)x(n+1) matrix must be taken...)
-    npoints = 0
+    npoints = 1
     if(inMatrix==["empty"]):
         print("What degree regression would you like to perform?")
         ndeg = int(input())
@@ -762,9 +764,31 @@ def nthRegression(restrict=0,inMatrix=["empty"],ndeg=0):
             outpoly += " + "
     if(restrict != 1):
         print("outpoly")
-    return [coeffs,outpoly]
+    R2 = 0
+    if(rcomp == 1):
+        def polyFunction(x):
+            y = 0
+            for i in range(ndeg+1):
+                y += coeffs[i][0]*(x**i)
+            return y
+        # Compute R^2 value (coefficient of determination)
+        # Begin by computing RSS (Residual Sum of Squares)
+        RSS = 0
+        for i in range(len(dataMt)):
+            RSS += (dataMt[i][1]-polyFunction(dataMt[i][0]))**2 # Sum of squares of residuals (errors between dataset values and model prediction)
+        # Now compute TSS (Total Sum of Squares)
+        TSS = 0
+        mu = 0 # Mean. Will sum all values (requiring starting value of 0) and then divide by number of values.
+        for i in range(len(dataMt)):
+            mu += dataMt[i][1]
+        mu /= len(dataMt)
+        for i in range(len(dataMt)):
+            TSS += (dataMt[i][1]-mu)**2 # Total sum of squares, which is proportional to variance. Sum of squares of differences between y-values and mean of y-values.
+        R2 = 1 - RSS/TSS # General definition for R^2 given RSS and TSS, see Coefficient of Determination on Wikipedia.
+        print("R^2 = %s" % (R2))
+    return [coeffs,outpoly,R2]
     
-def pwrRegression(restrict=0):
+def pwrRegression(restrict=0,rcomp=1):
     # Computes the closest power-regression (form of y=ax^b) fit to a dataset using logarithmic least-squares
     print("Would you like to use Matrix input (1) or Manual input (2)?")
     inmode = input()
@@ -788,10 +812,83 @@ def pwrRegression(restrict=0):
                 print("Program failure: Improper formatting of input.")
                 return
             lnMt.append([log(float(ptArr[0])),log(float(ptArr[1]))]) # Converts coordinates to (log(x),log(y)) to exploit fact that log(ax^b) = log(a) + b*log(x), which is linear.
-    # Need to take user input, like for nthRegression.
-    lnMt_reg = nthRegression(1,lnMt,1)[0] # Performs linear regression (n=1) on logarithmic dataset
+    lnMt_reg = nthRegression(1,lnMt,1,0)[0] # Performs linear regression (n=1) on logarithmic dataset
     lnMt_reg[0][0] = exp(lnMt_reg[0][0]) # Accounts for fact that "y-intercept" value is still in natural logarithm form by performing exponentiation
-    outeq = "y = %sx^(%s)" % (lnMt_reg[0][0],lnMt_reg[1][0]) # Readable format y = ax^(b)
+    outeq = "y = %sx^(%s)" % (Fraction(lnMt_reg[0][0]).limit_denominator(),Fraction(lnMt_reg[1][0]).limit_denominator()) # Readable format y = ax^(b)
     if(restrict != 1):
         print(outeq)
-    return [lnMt_reg,outeq]
\ No newline at end of file
+    R2 = 0
+    if(rcomp == 1):
+        a = lnMt_reg[0][0]
+        b = lnMt_reg[1][0]
+        def pwrFunction(x):
+            return a*(x**b)
+        # Compute R^2 value (coefficient of determination)
+        # Begin by computing RSS (Residual Sum of Squares)
+        RSS = 0
+        for i in range(len(lnMt)):
+            RSS += (exp(lnMt[i][1])-pwrFunction(exp(lnMt[i][0])))**2 # Sum of squares of residuals (errors between dataset values and model prediction)
+        # Now compute TSS (Total Sum of Squares)
+        TSS = 0
+        mu = 0 # Mean. Will sum all values (requiring starting value of 0) and then divide by number of values.
+        for i in range(len(lnMt)):
+            mu += exp(lnMt[i][1])
+        mu /= len(lnMt)
+        for i in range(len(lnMt)):
+            TSS += (exp(lnMt[i][1])-mu)**2 # Total sum of squares, which is proportional to variance. Sum of squares of differences between y-values and mean of y-values.
+        R2 = 1 - RSS/TSS # General definition for R^2 given RSS and TSS, see Coefficient of Determination on Wikipedia.
+        print("R^2 = %s" % (R2))
+    return [lnMt_reg,outeq,R2]
+
+def expRegression(restrict=0,rcomp=1):
+    # Computes the closest exponential-regression (form of y=ab^x) fit to a dataset using logarithmic least-squares
+    print("Would you like to use Matrix input (1) or Manual input (2)?")
+    inmode = input()
+    lnMt = []
+    if (int(inmode) == 1):
+        # I don't have anything to handle input matrices that are too small at the moment.
+        print("Enter index key of the dataset matrix to input.")
+        indx = input()
+        lnMt = index[str(indx)] # At the current stage of development, this needs to already be in (x,log(y)) format.
+    else:
+        print("How many datapoints would you like to input? You must have at least 3 datapoints. When entering datapoints use the format x,y.")
+        npoints = int(input())
+        if(npoints < 3):
+            print("Not enough datapoints. Failed.")
+            return
+        for i in range(npoints):
+            print("Enter datapoint at index %d:"%(i))
+            ptIn = input()
+            ptArr = ptIn.split(",")
+            if(len(ptArr) != 2):
+                print("Program failure: Improper formatting of input.")
+                return
+            lnMt.append([float(ptArr[0]),log(float(ptArr[1]))]) # Converts coordinates to (x,log(y)) to exploit fact that log(ab^x) = log(a) + x*log(b), which is linear.
+    lnMt_reg = nthRegression(1,lnMt,1,0)[0] # Performs linear regression (n=1) on logarithmic dataset
+    lnMt_reg[0][0] = exp(lnMt_reg[0][0]) # Account for a-value still being in natural logarithm form by exponentiating
+    lnMt_reg[1][0] = exp(lnMt_reg[1][0]) # Account for b-value logarithm form, as above
+    outeq = "y = %s*%s^x" % (Fraction(lnMt_reg[0][0]).limit_denominator(),Fraction(lnMt_reg[1][0]).limit_denominator()) # Readable format y = ax^(b)
+    if(restrict != 1):
+        print(outeq)
+    R2 = 0
+    if(rcomp == 1):
+        a = lnMt_reg[0][0]
+        b = lnMt_reg[1][0]
+        def expFunction(x):
+            return a*(b**x)
+        # Compute R^2 value (coefficient of determination)
+        # Begin by computing RSS (Residual Sum of Squares)
+        RSS = 0
+        for i in range(len(lnMt)):
+            RSS += (exp(lnMt[i][1])-expFunction(lnMt[i][0]))**2 # Sum of squares of residuals (errors between dataset values and model prediction)
+        # Now compute TSS (Total Sum of Squares)
+        TSS = 0
+        mu = 0 # Mean. Will sum all values (requiring starting value of 0) and then divide by number of values.
+        for i in range(len(lnMt)):
+            mu += exp(lnMt[i][1])
+        mu /= len(lnMt)
+        for i in range(len(lnMt)):
+            TSS += (exp(lnMt[i][1])-mu)**2 # Total sum of squares, which is proportional to variance. Sum of squares of differences between y-values and mean of y-values.
+        R2 = 1 - RSS/TSS # General definition for R^2 given RSS and TSS, see Coefficient of Determination on Wikipedia.
+        print("R^2 = %s" % (R2))
+    return [lnMt_reg,outeq,R2]
\ No newline at end of file
